program Demo;

{$define no_combine}

uses
  Windows, Messages,
  enZGLine{$ifndef no_combine}, OpenGL{$endif};

type
  TVerticle = record  
    Vertex: TVector3D;
    Tex1  : TVector2D;
    Tex2  : TVector2D;
  end;

  TTestType = (ttNone, ttFrust, ttOccl);
{$ifndef no_combine}
const
  GL_COMBINE_ARB = $8570;
  GL_COMBINE_RGB_ARB = $8571;
  GL_COMBINE_ALPHA_ARB = $8572;
  GL_SOURCE0_RGB_ARB = $8580;
  GL_SOURCE1_RGB_ARB = $8581;
  GL_SOURCE2_RGB_ARB = $8582;
  GL_SOURCE0_ALPHA_ARB = $8588;
  GL_SOURCE1_ALPHA_ARB = $8589;
  GL_SOURCE2_ALPHA_ARB = $858A;
  GL_OPERAND0_RGB_ARB = $8590;
  GL_OPERAND1_RGB_ARB = $8591;
  GL_OPERAND2_RGB_ARB = $8592;
  GL_OPERAND0_ALPHA_ARB = $8598;
  GL_OPERAND1_ALPHA_ARB = $8599;
  GL_OPERAND2_ALPHA_ARB = $859A;
  GL_RGB_SCALE_ARB = $8573;
  GL_ADD_SIGNED_ARB = $8574;
  GL_INTERPOLATE_ARB = $8575;
  GL_SUBTRACT_ARB = $84E7;
  GL_CONSTANT_ARB = $8576;
  GL_PRIMARY_COLOR_ARB = $8577;
  GL_PREVIOUS_ARB = $8578;
{$endif}

var
  mode: TMode =
    (ResX: 800;
     ResY: 600;
     bpp: 32;
     full: false;
     Freq: -1;
     AntiAliasing: true;
     Samples: -1;
     Anisotropy: true;
     v_sync: false);

  Qube: array[0..7] of TVerticle;
  Indexes: array[0..35] of WORD;

  V_Buf, I_Buf: TGLBuffer;
  Box: TBox;
  OccTest: TGLOccObj;
  T_Main, T_Sec: TGLTexture;
  angle: single = 0;
  visible: boolean = true;
  d: single = 1;

  test_type: TTestType = ttNone;
  filter: byte = 0;

function IntToStr(n: integer): string;
begin
  Str(n,result);
end;

procedure Msg(m: UINT; w, l: integer);
begin
  case m of
    WM_KEYUP:
    begin
      case w of
        VK_TAB:
          test_type := TTestType((byte(test_type)+1) mod 3);
        VK_SPACE: filter := (filter + 1) mod 4;
        ORD('P'): tex.ScreenShot('screen.bmp');
      end;
    end;
  end;
end;

procedure Init;
begin
  fsys.AddPacksDir('Data');

  Qube[0].Vertex := math3D.NewVector3D(-1,-1,-1);
  Qube[1].Vertex := math3D.NewVector3D(-1, 1,-1);
  Qube[2].Vertex := math3D.NewVector3D( 1, 1,-1);
  Qube[3].Vertex := math3D.NewVector3D( 1,-1,-1);
  Qube[4].Vertex := math3D.NewVector3D(-1,-1, 1);
  Qube[5].Vertex := math3D.NewVector3D(-1, 1, 1);
  Qube[6].Vertex := math3D.NewVector3D( 1, 1, 1);
  Qube[7].Vertex := math3D.NewVector3D( 1,-1, 1);

  Qube[0].Tex1 := math3D.NewVector2D(0,0);
  Qube[1].Tex1 := math3D.NewVector2D(1,0);
  Qube[2].Tex1 := math3D.NewVector2D(1,1);
  Qube[3].Tex1 := math3D.NewVector2D(0,1);
  Qube[4].Tex1 := math3D.NewVector2D(1,1);
  Qube[5].Tex1 := math3D.NewVector2D(0,1);
  Qube[6].Tex1 := math3D.NewVector2D(0,0);
  Qube[7].Tex1 := math3D.NewVector2D(1,0);

  Qube[0].Tex2 := math3D.NewVector2D(0,0);
  Qube[1].Tex2 := math3D.NewVector2D(1,0);
  Qube[2].Tex2 := math3D.NewVector2D(1,1);
  Qube[3].Tex2 := math3D.NewVector2D(0,1);
  Qube[4].Tex2 := math3D.NewVector2D(1,1);
  Qube[5].Tex2 := math3D.NewVector2D(0,1);
  Qube[6].Tex2 := math3D.NewVector2D(0,0);
  Qube[7].Tex2 := math3D.NewVector2D(1,0);

  Indexes[00] := 0;
  Indexes[01] := 1;
  Indexes[02] := 2;

  Indexes[03] := 0;
  Indexes[04] := 2;
  Indexes[05] := 3;

  Indexes[06] := 3;
  Indexes[07] := 2;
  Indexes[08] := 6;

  Indexes[09] := 3;
  Indexes[10] := 6;
  Indexes[11] := 7;

  Indexes[12] := 7;
  Indexes[13] := 6;
  Indexes[14] := 5;

  Indexes[15] := 7;
  Indexes[16] := 5;
  Indexes[17] := 4;

  Indexes[18] := 4;
  Indexes[19] := 5;
  Indexes[20] := 1;

  Indexes[21] := 4;
  Indexes[22] := 1;
  Indexes[23] := 0;

  Indexes[24] := 1;
  Indexes[25] := 5;
  Indexes[26] := 6;

  Indexes[27] := 1;
  Indexes[28] := 6;
  Indexes[29] := 2;

  Indexes[30] := 7;
  Indexes[31] := 3;
  Indexes[32] := 0;

  Indexes[33] := 7;
  Indexes[34] := 0;
  Indexes[35] := 4;

  V_Buf := vbo.NewID(224,BUF_DATA,@Qube[0],'V_Buf');     //224 = (3*4+2*4+2*4)*8
  I_Buf := vbo.NewID(72,BUF_ELEMENT,@Indexes[0],'I_Buf');//72  = 36*2

  T_Main := tex.LoadTexture('Data\Main.tga',true,true);
  T_Sec  := tex.LoadTexture('Data\sec.jpg',true,true);

  tex.Disable;
  tex.DisableTextures2D;

  box := math3D.NewBox(-1,-1,-1,1,1,1);
  OccTest := occ.NewID(box,-1);

  ogl.ClearColor(0.5,0,0.75,1);
end;

procedure Free;
begin

end;

procedure Update;
begin
  if inp.KeyDown(VK_ESCAPE) then eng.Quit;
  angle := angle + d;

  if angle > 150  then  d := -abs(d);
  if angle < -150 then d := abs(d);
end;

procedure Render;
begin
  ogl.Clear(true,true);
  ogl.Set3D(0.1,100.0,90,1.333);
// -----------------
  ogl.Translate(0,0,-5);
  ogl.Rotate(30,2,1,0.5);
  ogl.Blend(BLEND_NONE);

  vbo.Enable(V_Buf,I_Buf);
  vbo.EnableArray(ARR_VERTEX);
  vbo.EnableArray(ARR_TEXCOORD);
  vbo.EnableArray(ARR_TEXCOORD);

  vbo.VertexPointer(28,0,3);
  vbo.TexCoordPointer(28,12,2,0);
  vbo.TexCoordPointer(28,20,2,1);

  tex.EnableTextures2D;

  tex.AssignToTexUnit(T_Main,0,ENV_REPLACE);

  vbo.RenderBuffer(TRIANGLES,36,0);

  tex.DisableTextures2D;
  tex.Disable;
  ogl.Blend(BLEND_NONE);

  vbo.DisableArray(ARR_VERTEX);
  vbo.DisableArray(ARR_TEXCOORD);
  vbo.DisableArray(ARR_TEXCOORD);
  vbo.Disable;
  ogl.Rotate(-30,2,1,0.5);
  ogl.Translate(0,0,5);
//------------------------------------
  ogl.Rotate(angle,0,1,0);
  ogl.Translate(0,0,-10);
  ogl.Rotate(30,1,0,0);  
  ogl.SetColor(1,1,1,1);

  case test_type of
    ttNone: visible := true;
    ttFrust:
    begin
      math3D.BuildFrustum;
      visible := math3D.F_BoxInFrust(box);
    end;
    ttOccl:
    begin
      occ.Enable;
      occ.PostQuery(OccTest);
      occ.Disable;
      while not occ.IsQueryReady(OccTest) do
      begin  end;
      visible := occ.GetQueryResult(OccTest) <> 0;
    end;
  end;

  tex.FilterGroup(filter);

  if visible then
  begin
    ogl.Blend(BLEND_SUB);

    vbo.Enable(V_Buf,I_Buf);
    vbo.EnableArray(ARR_VERTEX);
    vbo.EnableArray(ARR_TEXCOORD);
    vbo.EnableArray(ARR_TEXCOORD);

    vbo.VertexPointer(28,0,3);
    vbo.TexCoordPointer(28,12,2,0);
    vbo.TexCoordPointer(28,20,2,1);

    tex.EnableTextures2D;

    tex.AssignToTexUnit(T_Main,0,ENV_REPLACE);
    tex.AssignToTexUnit(T_Sec,1,ENV_REPLACE);
    tex.EnableSphereTexCoords;
    {$ifndef no_combine}
    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE_ARB);
    glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB_ARB, GL_INTERPOLATE_ARB);
    glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_ALPHA_ARB, GL_REPLACE);

    glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_RGB_ARB,    GL_PREVIOUS_ARB);
    glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_RGB_ARB,   GL_SRC_COLOR);
    glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_ALPHA_ARB, GL_PREVIOUS_ARB);
    glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_ALPHA_ARB, GL_SRC_ALPHA);

    glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE1_RGB_ARB,    GL_TEXTURE);
    glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND1_RGB_ARB,   GL_SRC_COLOR);
    glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE1_ALPHA_ARB,  GL_TEXTURE);
    glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND1_ALPHA_ARB, GL_SRC_ALPHA);

    glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE2_RGB_ARB,    GL_PREVIOUS_ARB);
    glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND2_RGB_ARB,   GL_SRC_ALPHA);
    glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE2_ALPHA_ARB,  GL_TEXTURE);
    glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND2_ALPHA_ARB, GL_PREVIOUS_ARB);
    {$endif}

    vbo.RenderBuffer(TRIANGLES,36,0);

    tex.Disable;
    ogl.Blend(BLEND_NONE);

    vbo.DisableArray(ARR_VERTEX);
    vbo.DisableArray(ARR_TEXCOORD);
    vbo.DisableArray(ARR_TEXCOORD);
    vbo.Disable;
  end;
  tex.DisableTextures2D;

  ogl.Set2D(800,600);
  ogl.SetColor(1,1,1,1);
  ogl.glTextOut('FPS: '+IntToStr(eng.FPS),20,20,ogl.DebugFont);
  if visible
    then ogl.glTextOut('Visible: true',20,40,ogl.DebugFont)
    else ogl.glTextOut('Visible: false',20,40,ogl.DebugFont);
  case test_type of
    ttNone : ogl.glTextOut('[TAB]  Test type: None',20,60,ogl.DebugFont);
    ttFrust: ogl.glTextOut('[TAB]  Test type: Frustum test',20,60,ogl.DebugFont);
    ttOccl : ogl.glTextOut('[TAB]  Test type: Occlusion test',20,60,ogl.DebugFont);
  end;
  case filter of
    TF_NONE       : ogl.glTextOut('[SPACE]Filter: None',20,80,ogl.DebugFont);
    TF_BILINEAR   : ogl.glTextOut('[SPACE]Filter: Bilinear',20,80,ogl.DebugFont);
    TF_TRILINEAR  : ogl.glTextOut('[SPACE]Filter: Trilinear',20,80,ogl.DebugFont);
    TF_ANISOTROPY : ogl.glTextOut('[SPACE]Filter: Anisotropy',20,80,ogl.DebugFont);
  end;

  ogl.Swap;
end;

begin
  InitENZGLINE;//(mode);

  eng.SetProc(PROC_INIT,@init);
  eng.SetProc(PROC_FREE,@Free);
  eng.SetProc(PROC_UPDATE,@Update);
  eng.SetProc(PROC_RENDER,@Render);
  eng.SetProc(PROC_MSG,@Msg);

  eng.Loop(50);
end.
