unit enZGLine;

interface

uses
  windows;

const
  PROC_INIT   = $00;
  PROC_FREE   = $01;
  PROC_UPDATE = $02;
  PROC_RENDER = $03;
  PROC_MSG    = $04;

  FS_NEWFILE   = $00;
  FS_OPENFILE  = $01;

  FROM_BEGIN    = FILE_BEGIN;
  FROM_CURRENT  = FILE_CURRENT;
  FROM_END      = FILE_END;

  BLEND_NONE = $00;
  BLEND_SUB  = $01;
  BLEND_ADD  = $02;
  BLEND_MUL  = $03;

  TF_NONE       = $00;
  TF_BILINEAR   = $01;
  TF_TRILINEAR  = $02;
  TF_ANISOTROPY = $03;

  ENV_MODULATE = $2100;
  ENV_DECAL    = $2101;
  ENV_BLEND    = $0BE2;
  ENV_REPLACE  = $1E01;

  POINT           = 0;
  LINE            = 1;
  LINE_LOOP       = 2;
  LINE_STRIP      = 3;
  TRIANGLES       = 4;
  TRIANGLES_STRIP = 5;
  TRIANGLES_FAN   = 6;
  QUADS           = 7;
  QUADS_STRIP     = 8;
  POLYGON         = 9;

  FOG_LINEAR      = 0;
  FOG_EXP         = 1;
  FOG_EXP2        = 2;

  BUF_DATA        = 0;
  BUF_ELEMENT     = 1;

  ARR_VERTEX      = 0;
  ARR_NORMAL      = 1;
  ARR_TEXCOORD    = 2;
  ARR_COLOR       = 3;

  FACE_FRONT      = 0;
  FACE_BACK       = 1;
  FACE_NONE       = 2;

  POLY_WIRE       = 0;
  POLY_FILL       = 1;    

  GLSL_VERTEX   = 0;
  GLSL_FRAGMENT = 1;

  AL_LISTENER_NAME = 'LISTENER';

type
  TVector2D = record
  case integer of
    0: (x, y: single);
    1: (u, v: single);
    2: (s, t: single);
    3: (vec: array[0..1] of single);
  end;

  TVector3D = record
  case integer of
    0: (x, y, z: single);
    1: (u, v, f: single);
    2: (vec: array[0..2] of single);
  end;

  TPlane = record
  case integer of
    0: (x, y, z, dst: single);
    1: (normal: TVector3D; distance: single);
    2: (plane: array[0..3] of single);
  end;

  TMatrix4x4 = record
  case integer of
    0: (_11, _12, _13, _14,
        _21, _22, _23, _24,
        _31, _32, _33, _34,
        _41, _42, _43, _44: single);
    1: (m: array[0..3,0..3] of single);
  end;

  TQuaternion = record
  case integer of
    0: (x, y, z, w: single);
    1: (q: array[0..3] of single);
  end;

  TQuad = record
  case integer of
    0: (min, max: TVector2D);
    1: (x1, y1, x2, y2: single);
  end;

  TBox = record
  case integer of
    0: (min, max: TVector3D);
    1: (x1, y1, z1, x2, y2, z2: single);
  end;

  TMode = record
    ResX, ResY    : integer;
    bpp           : byte;
    full          : boolean;
    Freq          : integer;
    AntiAliasing  : boolean;
    Samples       : integer;
    Anisotropy    : boolean;
    v_sync        : boolean;
  end;

  TGLFont      = DWORD;
  TGLTexture   = DWORD;
  TGLModel     = DWORD;
  TGLBuffer    = DWORD;
  TGLSLShader  = DWORD;
  TGLSLProgram = DWORD;
  TALBuffer    = DWORD;
  TALSource    = DWORD;
  TGLOccObj    = DWORD;

  ILog = interface
    procedure print(message: string);                             overload;
    procedure print(ID: string; message: string);                 overload;
    procedure print(ID: string; message: string; status: string); overload;
    procedure msg(message: string);
    procedure EnableTiming;
    procedure DisableTiming;
  end;

  IWindow = interface
    procedure Mode;
    procedure ReturnDefaultMode;
    function GetMode: TMode;
    procedure SetMode(mode: TMode);
  end;

  IInput = interface
    procedure Reset;
    procedure LockMouse;
    procedure FreeMouse;
    procedure CursorShow;
    procedure CursorHide;
    function KeyDown(Key: byte): boolean;
    function MousePos: TPoint;
    function MouseDelta: TPoint;
    function MouseLDown: boolean;
    function MouseRDown: boolean;
    function MouseMDown: boolean;
    function MouseLocked: boolean;
  end;

  ICustomStream = interface
    function Read(var buf; size: DWORD): DWORD;
    function Write(var buf; size: DWORD): DWORD;               
    function Seek(offset: integer; method: DWORD): DWORD;      
    function Size: DWORD;                                      
    function Pos: DWORD;                                       
    function name: string;                                      overload;
    procedure name(NewID: string);                              overload;
    function CopyFrom(src: ICustomStream; size: DWORD): DWORD;
    function Valid: boolean;
    procedure SaveToFile(filename: string);    
  end;

  IFileStream = interface(ICustomStream)
    function Read(var buf; size: DWORD): DWORD;
    function Write(var buf; size: DWORD): DWORD;
    function Seek(offset: integer; method: DWORD): DWORD;
    function Size: DWORD;
    function Pos: DWORD;
    function CopyFrom(src: ICustomStream; size: DWORD): DWORD;
    function Valid: boolean;
    function name: string;                                     overload;
    procedure name(NewID: string);                             overload;
  end;

  IMemoryStream = interface(ICustomStream)
    function Read(var buf; size: DWORD): DWORD;
    function Write(var buf; size: DWORD): DWORD;
    function Seek(offset: integer; method: DWORD): DWORD;
    function Size: DWORD;
    function Pos: DWORD;
    function CopyFrom(src: ICustomStream; size: DWORD): DWORD;
    function Valid: boolean;
    function name: string;                                     overload;
    procedure name(NewID: string);                             overload;    
  end;

  IFileSystem = interface
    procedure AddPacksDir(dir: string);
    procedure Dispose;
    function OpenFile(name: string): ICustomStream; 
  end;

  IOpenGL = interface
    procedure Clear(color, depth: boolean);
    procedure Swap;

    procedure Set2D; overload;
    procedure Set2D(width, height: integer); overload;
    procedure Set3D(Pnear, Pfar, FOVY, Aspect: single);

    function FontCreate(fontname: PAnsiChar; w, h: integer): TGLFont;
    procedure glTextOut(text: string; X, Y: single; fontID: TGLFont);
    procedure FontDelete(font: TGLFont);
    function DebugFont: TGLFont;

    procedure SetColor(r, g, b: single); overload;
    procedure SetColor(r, g, b, a: single); overload;
    procedure SetVertex(x, y: single); overload;
    procedure SetVertex(x, y, z: single); overload;
    procedure SetTxCoord(u, v: single); overload;
    procedure SetTxCoord(u, v: single; tex_index: byte); overload;
    procedure SetNormal(x, y, z: single);
    procedure EnablePolygon(polygon_type: WORD);
    procedure DisablePolygon;

    procedure VertexPointer(vertex_size: integer; data: pointer);
    procedure ColorPointer(color_size: integer; data: pointer);
    procedure TexCoordPointer(texcoord_size: integer; data: pointer; tex_index: byte = 0);
    procedure NormalPointer(data: pointer);
    procedure DrawArray(polygon_type: WORD; count: integer);
    procedure DisableArray;

    procedure Enable(target: DWORD);
    procedure Disable(target: DWORD);
    procedure AntiAliasing(switch_on: boolean);

    procedure Blend(blend_type: byte);

    procedure Rotate(angle, rX, rY, rZ: single);
    procedure Translate(tx, ty, tz: single);
    procedure Scale(sx, sy, sz: single);
    procedure Look(fromX, fromY, fromZ, toX, toY, toZ, upX, upY, upZ: single);
    function GetModelViewMatrix: TMatrix4x4;
    function GetProjectionMatrix: TMatrix4x4;
    function GetTexMatrix: TMatrix4x4;
    procedure SetModelViewMatrix(m: TMatrix4x4);
    procedure SetProjectionMatrix(m: TMatrix4x4);
    procedure SetTexMatrix(m: TMatrix4x4);

    procedure ClearColor(r, g, b, a: single);
    procedure EnableFog(r, g, b, a, density, _start, _end: single; mode: byte);
    procedure DisableFog;

    procedure CullFacing(face: byte);
    procedure RenderMode(mode: byte);

    procedure EnableLight;
    procedure DisableLight;    
  end;

  ITexture = interface
    function NewID(data: pointer; width, height: integer; f: WORD; mipmap, clamp: boolean; name: string; group: integer = -1): TGLTexture;
    function GetID(name: string): TGLTexture;
    procedure ClearCash;

    function LoadTexture(filename: string; mipmap: boolean; clamp: boolean; group: integer = -1; name: string = ''): TGLTexture;
    procedure FreeTexture(ID: TGLTexture);

    procedure Enable(ID: TGLTexture);
    procedure Disable;

    procedure FilterGroup(ftype: byte; group: integer = -1);
    procedure FilterTex(ftype: byte; ID: TGLTexture);
    procedure DeleteGroup(group: integer);

    procedure AssignToTexUnit(texture: TGLTexture; tunit: byte; env_mode: WORD);
    procedure EnableSphereTexCoords;
    procedure DisableSphereTexCoords;
    procedure ScreenShot(filename: string);

    procedure EnableTextures2D;
    procedure DisableTextures2D;      
  end;

  IModel = interface
    function NewID(V_Count, F_Count, T_Count, Frames: integer; data: pointer; indexes: pointer; group: integer = -1; AA: boolean = false; name: string = ''; center: pointer = nil): TGLModel;
    function GetID(name: string): TGLModel;

    function LoadModel(filename: string; group: integer = -1; AA: boolean = false; name: string = ''): TGLModel;
    procedure FreeModel(var ID: TGLModel);

    procedure GroupAA(value: boolean; group: integer = -1);
    procedure ModelAA(ID: TGLModel; value: boolean);
    procedure DeleteGroup(group: integer = -1);

    procedure Render(ID: TGLModel; frame: integer);
  end;

  IVertexBufferObject = interface
    function NewID(data_size: integer; buf_type: byte;                name: string = ''): TGLBuffer; overload;
    function NewID(data_size: integer; buf_type: byte; data: pointer; name: string = ''): TGLBuffer; overload;
    function GetID(name: string): TGLBuffer;
    procedure FreeID(ID: TGLBuffer);
    procedure SubData(buf: TGLBuffer; buf_type: byte; offset, size: integer; data: pointer);

    procedure Enable(V_Buf, I_Buf: TGLBuffer);
    procedure Disable;

    procedure VertexPointer(stride: integer; start_pos: integer; elements_count: integer);
    procedure ColorPointer(stride: integer; start_pos: integer; elements_count: integer);
    procedure TexCoordPointer(stride: integer; start_pos: integer; elements_count: integer; tex_index: byte = 0);
    procedure NormalPointer(stride, start_pos: integer);
    procedure DisableArray(array_type: byte);
    procedure EnableArray(array_type: byte);
    procedure RenderBuffer(Primitiv_Type: DWORD; Indexes_count: integer; offset: integer);            overload;
    procedure RenderBuffer(Primitiv_type: DWORD; Indexes_count: integer; offset: integer; bpi: byte); overload;
  end;

  IVertexFragmentProgram = interface
    function NewID(name: string): TGLSLShader;
    function GetID(name: string): TGLSLShader;
    procedure FreeID(ID: TGLSLProgram);
    function CompileShader(filename: string; vfptype: byte): TGLSLShader;
    procedure AttachShaders(prog: TGLSLProgram; vsh, fsh: TGLSLShader);

    function LoadProgram(names: string; name: string = ''): TGLSLProgram;                overload;
    function LoadProgram(vsh, fsh: string; name: string = ''): TGLSLProgram;             overload;

    function GetUniformLocation(prog: TGLSLProgram; name: PAnsiChar): Integer;
    function GetAttribLocation(prog: TGLSLProgram; name: PAnsiChar): Integer;

    procedure SetUniform1f(prog: TGLSLProgram; name: PAnsiChar; v0: single);             overload;
    procedure SetUniform2f(prog: TGLSLProgram; name: PAnsiChar; v0, v1: single);         overload;
    procedure SetUniform3f(prog: TGLSLProgram; name: PAnsiChar; v0, v1, v2: single);     overload;
    procedure SetUniform4f(prog: TGLSLProgram; name: PAnsiChar; v0, v1, v2, v3: single); overload;
    procedure SetUniform1i(prog: TGLSLProgram; name: PAnsiChar; i0: Integer);            overload;

    procedure SetUniform1f(prog: TGLSLProgram; loc: Integer; v0: single);                overload;
    procedure SetUniform2f(prog: TGLSLProgram; loc: Integer; v0, v1: single);            overload;
    procedure SetUniform3f(prog: TGLSLProgram; loc: Integer; v0, v1, v2: single);        overload;
    procedure SetUniform4f(prog: TGLSLProgram; loc: Integer; v0, v1, v2, v3: single);    overload;
    procedure SetUniform1i(prog: TGLSLProgram; loc: Integer; i0: Integer);               overload;

    procedure SetAttrib1f(prog: TGLSLProgram; name: PAnsiChar; v0: single);              overload;
    procedure SetAttrib2f(prog: TGLSLProgram; name: PAnsiChar; v0, v1: single);          overload;
    procedure SetAttrib3f(prog: TGLSLProgram; name: PAnsiChar; v0, v1, v2: single);      overload;

    procedure SetAttrib1f(prog: TGLSLProgram; loc: Integer; v0: single);                 overload;
    procedure SetAttrib2f(prog: TGLSLProgram; loc: Integer; v0, v1: single);             overload;
    procedure SetAttrib3f(prog: TGLSLProgram; loc: Integer; v0, v1, v2: single);         overload;

    procedure Enable(prog: TGLSLProgram);
    procedure Disable;
  end;

  IOpenAL = interface
    procedure PlayMusic(filename: PAnsiChar; Loop: Boolean);
    procedure StopMusic;

    procedure Play(source: TALSource);
    procedure Stop(source: TALSource);
    procedure Pause(source: TALSource);

    procedure PlayGroup(group: integer = -1);
    procedure StopGroup(group: integer = -1);
    procedure PauseGroup(group: integer = -1);
    procedure GainGroup(gain: single; group: integer = -1);

    function NewSource(pos: TVector3D; vel: TVector3D; gain: Single; name: string; group: Integer = -1): TALSource;
    function GetSource(name: string): TALSource;
    function NewBuffer(name: String; group: Integer): TALBuffer;
    function GetBuffer(name: string): TALBuffer;

    procedure DeleteBuffer(ID: TALBuffer);
    procedure DeleteSource(ID: TALSource);

    function LoadFile(filename: String; group: Integer = -1; name: string = ''): TALBuffer;

    procedure AssignSource(source: TALSource; buffer: TALBuffer);

    procedure SetSourcePos(source: TALSource; pos: TVector3D);
    procedure SetSourceVel(source: TALSource; vel: TVector3D);
    procedure SetSourceLoop(source: TALSource; loop: boolean);
    procedure SetSourceGain(source: TALSource; gain: Single);

    procedure SetListenerPos(pos: TVector3D);
    procedure SetListenerVel(vel: TVector3D);
    procedure SetListenerOri(_to, _up: TVector3D);
    procedure SetListenerGain(gain: Single);
  end;

  IOcclusion = interface
    function NewID(box: TBox; group: integer = -1): TGLOccObj;
    procedure SetBox(ID: TGLOccObj; Box: TBox);
    function GetBox(ID: TGLOccObj): TBox;
    procedure FreeID(ID: TGLOccObj);

    procedure DeleteGroup(group: integer = -1);

    procedure Enable;
    procedure Disable;
    procedure PostQuery(ID: TGLOccObj);
    procedure PostQueryGroup(group: integer = -1);
    procedure PostQueryArray(IDs: array of TGLOccObj);

    function IsQueryReady(ID: TGLOccObj): boolean;
    function GetQueryResult(ID: TGLOccObj): DWORD;
  end;

  ICamera = interface
    procedure Init(p, a: TVector3D);
    procedure SetProj(FOV, aspect, P_near, P_far: single);
    procedure Update;

    procedure Step(v: TVector3D);
    procedure Move(p: TVector3D);
    procedure Rot(r: TVector3D);
    procedure Ang(a: TVector3D);
    function GetAngle: TVector3D;
    function GetPos: TVector3D;

    procedure StepForward(s: single);
    procedure StepBackward(s: single);
    procedure StepLeft(s: single);
    procedure StepRight(s: single);  
  end;

  IMath = interface
    //
    function NewVector2D(x, y: single): TVector2D;
    function NewVector3D(x, y, z: single): TVector3D;
    function NewPlane(normal: TVector3D; distance: single): TPlane;  overload;
    function NewPlane(nx, ny, nz, distance: single): TPlane;         overload;
    function NewQuaternion(x, y, z, w: single): TQuaternion;         overload;
    function NewQuaternion(v: TVector3D; angle: single): TQuaternion;overload;
    function NewQuad(min, max: TVector2D): TQuad;                    overload;
    function NewQuad(x1, y1, x2, y2: single): TQuad;                 overload;
    function NewBox(min, max: TVector3D): TBox;                      overload;
    function NewBox(x1, y1, z1, x2, y2, z2: single): TBox;           overload;

    //
    function V_Abs(v: TVector2D): single;                   overload;
    function V_Abs(v: TVector3D): single;                   overload;
    function V_Dot(v1, v2: TVector2D): single;              overload;
    function V_Dot(v1, v2: TVector3D): single;              overload;
    function V_Add(v1, v2: TVector2D): TVector2D;           overload;
    function V_Add(v1, v2: TVector3D): TVector3D;           overload;
    function V_Sub(v1, v2: TVector2D): TVector2D;           overload;
    function V_Sub(v1, v2: TVector3D): TVector3D;           overload;
    function V_Mul(v: TVector2D; f: single): TVector2D;     overload;
    function V_Mul(v: TVector3D; f: single): TVector3D;     overload;
    function V_Mul(v1, v2: TVector3D): TVector3D;           overload;
    function V_Mul(v: TVector3D; m: TMatrix4x4): TVector3D; overload;
    function V_Distance(v1, v2: TVector2D): single;         overload;
    function V_Distance(v1, v2: TVector3D): single;         overload;
    function V_Normalize(v: TVector2D): TVector2D;          overload;
    function V_Normalize(v: TVector3D): TVector3D;          overload;

    //
    function P_Normalize(p: TPlane): TPlane;

    //
    function M_Mul(m1, m2: TMatrix4x4): TMatrix4x4;       overload;
    function M_Mul(m: TMatrix4x4; f: single): TMatrix4x4; overload;
    function M_RotX(angle: single): TMatrix4x4;
    function M_RotY(angle: single): TMatrix4x4;
    function M_RotZ(angle: single): TMatrix4x4;
    function M_Transl(translation: TVector3D): TMatrix4x4;
    function M_Scale(scale: TVector3D): TMatrix4x4;

    //
    function B_Add(b: TBox; v: TVector3D): TBox;
    function B_Sub(b: TBox; v: TVector3D): TBox;

    //
    function Q_Add(b: TBox; v: TVector3D): TBox; overload;
    function Q_Sub(b: TBox; v: TVector3D): TBox; overload;

    //
    function Qt_GetMatrix(q: TQuaternion): TMatrix4x4;
    function Qt_RotateVector(v: TVector3D; q: TQuaternion): TVector3D;

    //  
    procedure BuildFrustum; overload;
    procedure BuildFrustum(proj, view: TMatrix4x4); overload;
    procedure BuildFrustum(proj_view: TMatrix4x4);  overload;
    function F_BoxInFrust(box: TBox): boolean;
  end;

  IEngine = interface
    procedure SetProc(ID: byte; proc: pointer);
    function GetLog: ILog;
    function GetWnd: IWindow;
    function GetInp: IInput;
    function GetFSys: IFileSystem;
    function GetOGL: IOpenGL;
    function GetTex: ITexture;
    function GetMdl: IModel;
    function GetVBO: IVertexBufferObject;
    function GetVFP: IVertexFragmentProgram;
    function GetOCC: IOcclusion;
    function GetOAL: IOpenAL;
    function GetCam: ICamera;
    function GetMath: IMath;
    function FPS: integer;
    procedure Loop(UPS: integer);
    procedure Quit;
  end;

function enZGLineInit: IEngine; external 'enZGLine.dll';
function enZGLineInit_mode(mode: TMode): IEngine; external 'enZGLine.dll';
function CreateFileStream(filename: string; openflag: byte): IFileStream; external 'enZGLine.dll';
function CreateMemoryStream: IMemoryStream; external 'enZGLine.dll';
procedure DeleteFileStream  (stream: IFileStream); external 'enZGLine.dll';
procedure DeleteMemoryStream(stream: IMemoryStream); external 'enZGLine.dll';

var
  eng: IEngine                = nil;
  log: ILog                   = nil;
  wnd: IWindow                = nil;
  fsys: IFileSystem           = nil;
  inp: IInput                 = nil;
  ogl: IOpenGL                = nil;
  tex: ITexture               = nil;
  mdl: IModel                 = nil;
  vbo: IVertexBufferObject    = nil;
  vfp: IVertexFragmentProgram = nil;
  occ: IOcclusion             = nil;
  cam: ICamera                = nil;
  oal: IOpenAL                = nil;
  math3D: IMath               = nil;

procedure InitENZGLINE; overload;
procedure InitENZGLINE(mode: TMode); overload;

implementation

procedure InitENZGLINE(mode: TMode); overload;
begin
  eng := enZGLineInit_mode(mode);
  log := eng.GetLog;
  inp := eng.GetInp;
  wnd := eng.GetWnd;
  fsys := eng.GetFSys;
  tex := eng.GetTex;
  ogl := eng.GetOGL;
  mdl := eng.GetMdl;
  vbo := eng.GetVBO;
  vfp := eng.GetVFP;
  occ := eng.GetOCC;
  cam := eng.GetCam;
  math3D := eng.GetMath;
  oal := eng.GetOAL;
end;

procedure InitENZGLINE;
begin
  eng := enZGLineInit;
  log := eng.GetLog;
  inp := eng.GetInp;
  wnd := eng.GetWnd;
  fsys := eng.GetFSys;
  tex := eng.GetTex;
  ogl := eng.GetOGL;
  mdl := eng.GetMdl;
  vbo := eng.GetVBO;
  vfp := eng.GetVFP;
  occ := eng.GetOCC;
  cam := eng.GetCam;  
  math3D := eng.GetMath;
  oal := eng.GetOAL;
end;

initialization
  eng  := nil;
  log  := nil;
  wnd  := nil;
  fsys := nil;
  inp  := nil;
  ogl  := nil;
  tex  := nil;
  mdl  := nil;
  vbo  := nil;
  vfp  := nil;
  occ  := nil;
  cam  := nil;
  oal  := nil;
  math3D:= nil;

finalization
  math3D:= nil;
  oal  := nil;
  cam  := nil;
  occ  := nil;
  vfp  := nil;
  vbo  := nil;
  mdl  := nil;
  tex  := nil;
  ogl  := nil;
  inp  := nil;
  fsys := nil;
  wnd  := nil;
  log  := nil;
  eng  := nil;

end.
